var request = require("request"),
    baseURI = "http://api.yifysubtitles.com/subs/",
    downloadURI = "http://www.yifysubtitles.com",
    Q = require('q'),
    _ = require('underscore');


module.exports = {
    syntaxUrl: function(imdb) {
        imdb = (imdb.match("tt")) ? imdb : "tt" + imdb;
        return imdb;
    },
    get: function(imdb) {
        var deferred = Q.defer();
        request({
            url: baseURI + imdb,
            json: true
        }, function(err, res, data) {
            if (err || res.statusCode >= 400 || !data || !data.success) {
                return deferred.reject(err);
            } else {
                return deferred.resolve(data);
            }
        });
        return deferred.promise;
    },
    formatForUse: function(data, imdb) {
        var Subs = {},
            movieSubs = [],
            langs = [],
            self = this;
        _.each(data.subs[imdb], function(langsData, lang) {
            var langData = _.sortBy(langsData, function(s) {
                    return s.rating;
                }).reverse(),
                langPrefix = self.languageMapping[lang];
            langs.push(langPrefix);
            movieSubs[langPrefix] = [];

            _.each(langData, function(Data) {
                movieSubs[langPrefix].push({
                    rating: Data.rating,
                    url: downloadURI + Data.url
                })
            });
        });
        Subs.languages = langs;
        Subs.subs = movieSubs;
        return Subs;
    },
    getSubs: function(imdb) {
        var def = Q.defer(),
            self = this,
            imdb = self.syntaxUrl(imdb);

        self.get(imdb).then(function(data) {
            def.resolve(self.formatForUse(data, imdb));
        }).catch(function(error) {
            def.reject(error);
        });
        return def.promise;
    },
    filter: function (data, arr){
    	var result, subs = [];

    	for(sub in data.subs){
    		if(_.contains(arr, sub)) subs[sub] = data.subs[sub];
    	}

    	data.languages = arr;
    	data.subs = subs;

    	return data;
    },
    getlanguageName : function(prefix){
    	var langs = this.languageMapping;
    	for(lang in langs){
    		if (langs[lang] === prefix) return lang;
    	}
    },
    languageMapping: {
        'albanian': 'sq',
        'arabic': 'ar',
        'bengali': 'bn',
        'brazilian-portuguese': 'pt-br',
        'bulgarian': 'bg',
        'bosnian': 'bs',
        'chinese': 'zh',
        'croatian': 'hr',
        'czech': 'cs',
        'danish': 'da',
        'dutch': 'nl',
        'english': 'en',
        'estonian': 'et',
        'farsi-persian': 'fa',
        'finnish': 'fi',
        'french': 'fr',
        'german': 'de',
        'greek': 'el',
        'hebrew': 'he',
        'hungarian': 'hu',
        'indonesian': 'id',
        'italian': 'it',
        'japanese': 'ja',
        'korean': 'ko',
        'lithuanian': 'lt',
        'macedonian': 'mk',
        'malay': 'ms',
        'norwegian': 'no',
        'polish': 'pl',
        'portuguese': 'pt',
        'romanian': 'ro',
        'russian': 'ru',
        'serbian': 'sr',
        'slovenian': 'sl',
        'spanish': 'es',
        'swedish': 'sv',
        'thai': 'th',
        'turkish': 'tr',
        'urdu': 'ur',
        'ukrainian': 'uk',
        'vietnamese': 'vi'
    }
}
